==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/herk/zherk_k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

#undef SYRK_KERNEL_U
#undef SYRK_KERNEL_L


#ifdef COMPLEX
#ifndef CONJ
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UN
#define SYRK_KERNEL_L xherk_kernel_LN
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UN
#define SYRK_KERNEL_L zherk_kernel_LN
#else
#define SYRK_KERNEL_U cherk_kernel_UN
#define SYRK_KERNEL_L cherk_kernel_LN
#endif
#define GEMM_KERNEL GEMM_KERNEL_R
#else
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UC
#define SYRK_KERNEL_L xherk_kernel_LC
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UC
#define SYRK_KERNEL_L zherk_kernel_LC
#else
#define SYRK_KERNEL_U cherk_kernel_UC
#define SYRK_KERNEL_L cherk_kernel_LC
#endif
#define GEMM_KERNEL GEMM_KERNEL_L
#endif
#endif

#ifndef SWITCHING
#define SWITCHING	(GEMM_P *  2)
#define GEMM_PP		(GEMM_P *  1)
#endif

#define RRR  MAX(GEMM_UNROLL_M, GEMM_UNROLL_N)

int CNAME(long dummy1, long n, long k, FLOAT alpha_r, FLOAT alpha_i,
	  FLOAT *a, long lda, 
	  FLOAT *dummy2, long dummy3,
	  FLOAT *c, long ldc, FLOAT *buffer, long from, long to){

  long ls, is, js;
  long min_l, min_i, min_j;
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

#ifndef PPC440
  FLOAT *sa = (FLOAT *)((void *)buffer + GEMM_BUFFER_A_OFFSET);
  FLOAT *sb = (FLOAT *)((void *)buffer + GEMM_BUFFER_B_OFFSET
		 + ((GEMM_P * GEMM_Q * COMPSIZE * SIZE + GEMM_BUFFER_A_OFFSET * COMPSIZE + GEMM_ALIGN) & ~GEMM_ALIGN));
  FLOAT *subbuffer = (FLOAT *)((void *)buffer + BUFFER_LIMIT);
#else
  extern FLOAT *sa, *sb;
  FLOAT *subbuffer = malloc(GEMM_P * GEMM_Q * COMPSIZE * SIZE);
#endif

#if (GEMM_UNROLL_M == GEMM_UNROLL_N) && !defined(PPC440)

  for(ls = 0; ls < k; ls += GEMM_Q){
    min_l = k - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = from; js < to; js += GEMM_R){
      min_j = to - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
      
#ifndef TRANS
      GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
      GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif
      
      for(is = 0; is < n; is += GEMM_P){
	min_i = n - is;
	if (min_i > GEMM_P) min_i = GEMM_P;
	
	if ((is >= js) && (is + min_i <= js + min_j)) {

	  aa = sb + min_l * (is - js) * COMPSIZE;

	} else {
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	aa = sa;
	}
	
#ifndef LOWER
	SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
		      aa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, subbuffer);
#else
	SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
		      aa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, subbuffer);
#endif

      } /* end of ls */
    } /* end of is */
  } /* end of js */

#else
#if defined(SWITCHING)
  if (n >= SWITCHING) {
#endif
  for(ls = 0; ls < k; ls += GEMM_Q){
    min_l = k - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = from; js < to; js += GEMM_R){
      min_j = to - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
      
#ifndef TRANS
      GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
      GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif
      
      for(is = 0; is < n; is += GEMM_P){
	min_i = n - is;
	if (min_i > GEMM_P) min_i = GEMM_P;
	
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	
#ifndef LOWER
	SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
		      sa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, subbuffer);
#else
	SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
		      sa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, subbuffer);
#endif

      } /* end of ls */
    } /* end of is */
  } /* end of js */
#if defined(SWITCHING)
  } else {
    for(ls = 0; ls < k; ls += GEMM_Q){
      min_l = k - ls;
      if (min_l > GEMM_Q) min_l = GEMM_Q;
      
      for(is = 0; is < n; is += GEMM_PP){
	min_i = n - is;
	if (min_i > GEMM_PP) min_i = GEMM_PP;
	
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	
	for(js = from; js < to; js += RRR){
	  min_j = to - js;
	  if (min_j > RRR) min_j = RRR;
	  
#ifndef TRANS
	  GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
	  GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif

#ifndef LOWER
	  SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
			sa, sb, 
			c + (is + js * ldc) * COMPSIZE, ldc,
			is - js, subbuffer);
#else
	  SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
			sa, sb, 
			    c + (is + js * ldc) * COMPSIZE, ldc,
			is - js, subbuffer);
#endif
	  
	} /* end of ls */
      } /* end of is */
    } /* end of js */
  }
#endif
#endif
  
#ifdef PPC440
  free(subbuffer);
#endif

  return 0;
}

==> GotoBLAS-1.19.tar.gz: GotoBLAS/level3/herk/zherk_k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

#undef SYRK_KERNEL_U
#undef SYRK_KERNEL_L


#ifdef COMPLEX
#ifndef CONJ
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UN
#define SYRK_KERNEL_L xherk_kernel_LN
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UN
#define SYRK_KERNEL_L zherk_kernel_LN
#else
#define SYRK_KERNEL_U cherk_kernel_UN
#define SYRK_KERNEL_L cherk_kernel_LN
#endif
#define GEMM_KERNEL GEMM_KERNEL_R
#else
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UC
#define SYRK_KERNEL_L xherk_kernel_LC
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UC
#define SYRK_KERNEL_L zherk_kernel_LC
#else
#define SYRK_KERNEL_U cherk_kernel_UC
#define SYRK_KERNEL_L cherk_kernel_LC
#endif
#define GEMM_KERNEL GEMM_KERNEL_L
#endif
#endif

#ifndef SWITCHING
#define SWITCHING	(GEMM_P *  2)
#define GEMM_PP		(GEMM_P *  1)
#endif

#define MAXMN MAX((GEMM_UNROLL_M), (GEMM_UNROLL_N))

int CNAME(long dummy1, long n, long k, FLOAT alpha_r, FLOAT alpha_i,
	  FLOAT *a, long lda, 
	  FLOAT *dummy2, long dummy3,
	  FLOAT *c, long ldc, FLOAT *sa, FLOAT *sb, long from, long to){

  long ls, is, js;
  long min_l, min_i, min_j;

  FLOAT *sub = (FLOAT *)alloca(MAXMN * MAXMN * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

#if (GEMM_UNROLL_M == GEMM_UNROLL_N) && !defined(PPC440)

  for(ls = 0; ls < k; ls += GEMM_Q){
    min_l = k - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = from; js < to; js += GEMM_R){
      min_j = to - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
      
#ifndef TRANS
      GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
      GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif
      
      for(is = 0; is < n; is += GEMM_P){
	min_i = n - is;
	if (min_i > GEMM_P) min_i = GEMM_P;
	
	if ((is >= js) && (is + min_i <= js + min_j)) {

	  aa = sb + min_l * (is - js) * COMPSIZE;

	} else {
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	aa = sa;
	}
	
#ifndef LOWER
	SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
		      aa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, sub);
#else
	SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
		      aa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, sub);
#endif

      } /* end of ls */
    } /* end of is */
  } /* end of js */

#else
#if defined(SWITCHING)
  if (n >= SWITCHING) {
#endif
  for(ls = 0; ls < k; ls += GEMM_Q){
    min_l = k - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = from; js < to; js += GEMM_R){
      min_j = to - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
      
#ifndef TRANS
      GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
      GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif
      
      for(is = 0; is < n; is += GEMM_P){
	min_i = n - is;
	if (min_i > GEMM_P) min_i = GEMM_P;
	
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	
#ifndef LOWER
	SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
		      sa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, sub);
#else
	SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
		      sa, sb, 
		      c + (is + js * ldc) * COMPSIZE, ldc,
		      is - js, sub);
#endif

      } /* end of ls */
    } /* end of is */
  } /* end of js */
#if defined(SWITCHING)
  } else {
    for(ls = 0; ls < k; ls += GEMM_Q){
      min_l = k - ls;
      if (min_l > GEMM_Q) min_l = GEMM_Q;
      
      for(is = 0; is < n; is += GEMM_PP){
	min_i = n - is;
	if (min_i > GEMM_PP) min_i = GEMM_PP;
	
#ifndef TRANS
	GEMM_ITCOPY(min_l, min_i, a + (is + ls * lda) * COMPSIZE, lda, sa);
#else
	GEMM_INCOPY(min_l, min_i, a + (ls + is * lda) * COMPSIZE, lda, sa);
#endif
	
	for(js = from; js < to; js += MAXMN){
	  min_j = to - js;
	  if (min_j > MAXMN) min_j = MAXMN;
	  
#ifndef TRANS
	  GEMM_OTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, sb);
#else
	  GEMM_ONCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, sb);
#endif

#ifndef LOWER
	  SYRK_KERNEL_U(min_i, min_j, min_l, alpha_r,
			sa, sb, 
			c + (is + js * ldc) * COMPSIZE, ldc,
			is - js, sub);
#else
	  SYRK_KERNEL_L(min_i, min_j, min_l, alpha_r, 
			sa, sb, 
			    c + (is + js * ldc) * COMPSIZE, ldc,
			is - js, sub);
#endif
	  
	} /* end of ls */
      } /* end of is */
    } /* end of js */
  }
#endif
#endif
  
  return 0;
}
